@@ -2,7 +2,9 @@
 提供兼容 s3 的常用 file system 操作 
 """
 import os
+import sys
 import uuid
+import shutil
 import megfile
 import pathlib
 import builtins
@@ -27,7 +29,7 @@ os_abspath = os.path.abspath
 
 def abspath_for_s3(self):
     """
-    To support 
+    To support
 
     >>> smart_abspath('s3://a/b/../c')
     's3://a/c'
@@ -60,7 +62,7 @@ class ramfs:
         装饰器, 装饰某个函数后, 使其兼容 s3 path::
             img = ramfs(imread)(s3_path)
             ramfsw(imsave)(s3_path, img)
-        
+
         原理: 会将对应的 oss 文件拷贝到内存文件 (/dev/shm 下), 对应函数在内存文件上操作完成后再同步回 oss
         arg:
             None: 自动找到第一个是 (str, bytes, pathlib.Path) 的变量作为 path
@@ -120,16 +122,17 @@ class ramfs:
     @staticmethod
     def test():
         from skimage import data
-        from boxx import imsave, imread, show, mg
+        from boxx import imsave, imread, show, g
 
         img = data.coffee()
+        # img = data.astronaut()
         path = "s3://yl-share/tmp/a.jpg"
         ramfsw(imsave)(path, img)
         show - ramfs(imread)(path)
         url = path.replace("s3://", "https://oss.iap.hh-b.brainpp.cn/")
         print(" S3:", path)
         print("URL:", url)
-        mg()
+        g()
 
 
 class ramfsw(ramfs):
@@ -176,24 +179,24 @@ def compat_mode():
         compat_mode()
 
         import boxx
-    
+
     - 被侵入式替换的函数记录在 brainpp_yl.fs.std_funcs, 不支持 isdir (有神奇BUG)
     """
-    skimage.io.imread = imread
-    skimage.io.imsave = imsave
+    if getattr(compat_mode, "in_compat_mode", None):
+        return
 
     # 在 IPython 中第二次 run 的时候会导致 len not found 错误
-    # os.path.isdir = isdir
+    os.path.isdir = isdir
     for k, d in std_funcs.items():
         setattr(d["module"], k, d["compat_func"])
     removeimp("boxx")
-    # import boxx
-    # boxx.loadjson = ramfs(boxx.loadjson)
-    # boxx.savejson = ramfsw(boxx.savejson)
-    # boxx.loadCoco = ramfs(boxx.loadCoco)
-    # boxx.saveCoco = ramfsw(boxx.saveCoco)
-
     builtins.open = open
+    builtins.__import__ = __import__
+    for name in sys.modules:
+        if name in import_smart_module_register:
+            for import_smart_func in import_smart_module_register[name]:
+                import_smart_func(sys.modules[name])
+    compat_mode.in_compat_mode = True
 
 
 listdir = def_new_path_func(os, "listdir")
@@ -206,10 +209,29 @@ relpath = def_new_path_func(os.path, "relpath")
 realpath = def_new_path_func(os.path, "realpath")
 isfile = def_new_path_func(os.path, "isfile")
 isabs = def_new_path_func(os.path, "isabs")
+
+rmdir = def_new_path_func(os, "rmdir", "remove")
+rmtree = def_new_path_func(shutil, "rmtree", "remove")
 glob = def_new_path_func(glob_std, "glob")
 
+# Add all smart to def_new_path_func
+dir_os = dir(os)
+dir_os_path = dir(os.path)
+for smart_name in dir(megfile):
+    name = smart_name.replace("smart_", "")
+    if not smart_name.startswith("smart_") or name in ["path"]:
+        continue
+    if name in dir_os:
+        module = os
+    elif name in dir_os_path:
+        module = os.path
+    else:
+        continue
+    globals()[name] = def_new_path_func(module, name)
+
+
 # 在 IPython 中第二次 run 的时候会导致 len not found 错误
-# isdir = def_new_path_func(os.path, "isdir")
+isdir = def_new_path_func(os.path, "isdir")
 isdir_std = os.path.isdir
 
 
@@ -220,9 +242,10 @@ def isdir(s):
 
 
 imread = ramfs(skimage.io.imread, 0)
-imsave = ramfsw(skimage.io.imsave, 0)
+imsave = ramfs(skimage.io.imsave, 0, mode="w")
 
 builtins_open = builtins.open
+builtins_import = builtins.__import__
 
 
 @wraps(builtins_open)
@@ -234,6 +257,45 @@ def open(file, *args, **argkv):
         return smart_open(file, *args, **argkv)
 
 
+class FuncRegister(dict):
+    def __call__(self, module_name):
+        assert isinstance(
+            module_name, str
+        ), f"Should `@import_smart_module_register('{module_name}')`"
+
+        def append_func(func):
+            if module_name not in self:
+                self[module_name] = []
+            self[module_name].append(func)
+            return func
+
+        return append_func
+
+
+import_smart_module_register = FuncRegister()
+
+
+@import_smart_module_register("skimage.io")
+def import_smart_skimage_io(skimage_io):
+    skimage_io.imread = imread
+    skimage_io.imsave = imsave
+
+
+@import_smart_module_register("cv2")
+def import_smart_cv2(cv2):
+    cv2.imread = ramfs(cv2.imread)
+    cv2.imwrite = ramfs(cv2.imwrite, mode="w")
+
+
+@wraps(builtins_import)
+def __import__(name, *args, **argkws):
+    module = builtins_import(name, *args, **argkws)
+    if name in import_smart_module_register:
+        for import_smart_func in import_smart_module_register[name]:
+            import_smart_func(module)
+    return module
+
+
 copy = smart_copy
 
 if __name__ == "__main__":
