@@ -17,12 +17,30 @@ def split_keys_by_replica(keys):
     """
     `bpp.split_keys_by_replica`: 在 replica 模式下, 自动划分 keys (数据集), 兼容非 replica 模式
     """
+    import hashlib
     from .env import replica_idx, replica_total
 
-    keys = sorted(keys)
+    if not len(keys):
+        return keys
+    keys = sorted(keys)  # 没有 replica 的 worker sorted 后方便查看进展
     if replica_total == 1:
         return keys
-    keys = keys[replica_idx::replica_total]
+    if (
+        len(keys) / replica_total > 10
+        and isinstance(keys[-1], str)
+        and not keys[-1].isdigit()
+    ):
+        # 对不是 int 的 str, 使用 md5 来分配计算 key, 尤其是 nori id
+        # 避免 keys 长度变化的时候会导致处理重复和遗漏
+        get_hash_int = lambda key: int(
+            hashlib.md5((str(key) + "-bppyl_split_salt").encode("utf-8")).hexdigest()[
+                -7:
+            ],
+            16,
+        )
+        keys = [key for key in keys if get_hash_int(key) % replica_total == replica_idx]
+    else:
+        keys = keys[replica_idx::replica_total]
     shuffle(keys)
     return keys
 
@@ -84,7 +102,7 @@ class NoriPickleAndPut:
     def __exit__(self, typee, value, traceback):
         self.w.__exit__(typee, value, traceback)
 
-    def pickle_and_put(self, data, filename=None):
+    def pickle_and_put(self, data, filename=""):
         import pickle
 
         return self.w.put(pickle.dumps(data), filename=filename)
